@@ -224,14 +224,18 @@ dataOrganize$set('public', 'makeMultinom', function(multinomVars, return, oldVar
 #' @param temporalname Name of the temporal variable used in the model.
 #' @param spatial Logical: are spatial effects run in the model.
 #' @param marksspatial Logical: should spatial fields be included for the marks.
+#' @param speciesintercept Logical: should specific species intercept terms be created for the species.
+#' @param speciesenvironment Logical: should species specific environmental terms be created.
 #' @param intercept Logical: are intercepts run in the model.
 #' @param markintercept Logical: are intercepts run for the marks in the model.
 #' @param pointcovs Name of the point covariates.
 #' @param speciesspatial Logical: should the species have spatial fields.
+#' @param speciesindependent Logical: make independent species effects.
 
 dataOrganize$set('public', 'makeFormulas', function(spatcovs, speciesname,
-                                                    paresp, countresp, marks, marksspatial,
-                                                    spatial, intercept, temporalname,
+                                                    paresp, countresp, marks, marksspatial, 
+                                                    speciesintercept, speciesenvironment,
+                                                    spatial, intercept, temporalname, speciesindependent,
                                                     markintercept, pointcovs, speciesspatial) {
   
   #if (length(self$multinomVars) != 0) marks[marks %in% self$multinomVars] <- paste0(marks[marks %in% self$multinomVars],'_response')
@@ -298,9 +302,20 @@ dataOrganize$set('public', 'makeFormulas', function(spatcovs, speciesname,
           if (!is.null(speciesIn)) {
              if (pointsResponse[[response]][j] %in% c('geometry', paresp, countresp)) {
               ##Change this part ot the speciesIn: not sure what the one below does...
-              if (!is.null(speciesspatial)) speciesspat <- paste0(do.call(paste0, expand.grid(paste0(speciesIn,'_'),
+              if (!is.null(speciesspatial)) {
+                
+                if (speciesspatial == 'shared') speciesspat <- 'speciesShared'
+                else {
+                
+                  
+                if (speciesindependent) speciesspat <- paste0(speciesIn,'_spatial')
+                else speciesspat <- paste0(do.call(paste0, expand.grid(paste0(speciesIn,'_'),
                                                                                               names(self$Data)[[dataset]])),
                                                                   '_spatial') ## new argument called speciesSpatial??
+                
+                }
+                
+              }
               else speciesspat <- NULL
 
             
@@ -310,9 +325,17 @@ dataOrganize$set('public', 'makeFormulas', function(spatcovs, speciesname,
             #  else spat <- NULL
               
             #}
+            if (!is.null(speciesintercept)) {
+              
+              if (speciesintercept) int <- paste0(speciesname, '_intercepts')
+              else int <- paste0(speciesIn,'_intercept') 
+              
+            } else if (intercept) {
+              
+              int <- paste0(names(self$Data)[[dataset]],'_intercept') 
+              
+            } else int <- NULL  
             
-            if (intercept) int <- paste0(speciesIn,'_intercept') 
-            else int <- NULL
              }
             else {
               
@@ -351,7 +374,12 @@ dataOrganize$set('public', 'makeFormulas', function(spatcovs, speciesname,
           
           if (!is.null(spatcovs)) {
             
-            if (!is.null(speciesname)) covs <- paste0(speciesIn, '_', spatcovs)
+            if (!is.null(speciesname)) {
+              
+              if (speciesenvironment) covs <- paste0(speciesIn, '_', spatcovs)
+              else covs <- spatcovs
+              
+            }
             else covs <- spatcovs
             
           }
@@ -441,10 +469,13 @@ dataOrganize$set('public', 'makeFormulas', function(spatcovs, speciesname,
 #' @param temporalname Name of the temporal variable used in the model.
 #' @param marksspatial Logical: should spatial fields be included for the marks.
 #' @param marksintercept Logical: should intercepts be included for the marks.
+#' @param speciesintercept: Logical: should intercept terms be created for each species.
 #' @param numtime Number of time increments included in the model.
 #' @param speciesspatial Logical: Should the species be run with spatial fields.
+#' @param speciesenvironment Logical: Should the species have their own environmental effects.
 #' @param offsetname Name of the offset column in the datasets.
 #' @param copymodel List of the hyper parameters for the \code{copy} model.
+#' @param speciesindependent Logical: should species effects be made independent.
 
 dataOrganize$set('public', 'makeComponents', function(spatial, intercepts, 
                                                       datanames, marks, speciesname,
@@ -454,9 +485,12 @@ dataOrganize$set('public', 'makeComponents', function(spatial, intercepts,
                                                       marksintercept,
                                                       temporalname,
                                                       speciesspatial,
+                                                      speciesenvironment,
+                                                      speciesintercept,
                                                       numtime,temporalmodel,
                                                       offsetname,
-                                                      copymodel) {
+                                                      copymodel,
+                                                      speciesindependent) {
   ##Copy for marks fields???
   if (length(self$SpeciesInData) != 0) species <- unique(unlist(self$SpeciesInData))
   else species = NULL
@@ -494,7 +528,8 @@ dataOrganize$set('public', 'makeComponents', function(spatial, intercepts,
     if (!is.null(speciesspatial)) { ## Then if copy or individual ...
       
       #if (length(speciesspatial) > 0) {
-      
+      if (speciesspatial == 'shared') speciesSpat <- 'speciesShared(main = geometry, model = speciesField)'
+      else {
       #speciesSpat <- paste0(species,'_spatial(main = coordinates, model = ',paste0(species,'_spdeModel'),')', collapse = ' + ')
       
       #} 
@@ -507,6 +542,9 @@ dataOrganize$set('public', 'makeComponents', function(spatial, intercepts,
         #speciesSpat <- paste0(speciesname, '_spatial(main = coordinates, model = speciesModel, group = ',speciesname,', ngroup = ', numspecies,')')
         if (speciesspatial == 'individual' || length(species) == 1) {
           
+          if (speciesindependent) speciesSpat <- paste0(species,'_spatial(main = geometry, model = ',paste0(species,'_field)'))
+          else {
+          
           speciesComps <- list()
           
           for (spec in species) {
@@ -521,9 +559,20 @@ dataOrganize$set('public', 'makeComponents', function(spatial, intercepts,
           
           speciesSpat <- unlist(speciesComps)
           
+          }
+          
         }
         else {
           
+          if (speciesindependent) {
+            
+            speciesOne <- paste0(species[1],'_spatial(main = geometry, model = ',paste0(species[1],'_field)'))
+            if (length(species) > 1) speciesOther <- paste0(species[-1],'_spatial(main = geometry, copy = \"', species[1],'_spatial\",  hyper = list(beta = list(fixed = FALSE)))')
+            else speciesOther <- NULL
+          
+          }
+          else {
+          
           speciesOne <- list()
           speciesOther <- list()
           
@@ -557,11 +606,12 @@ dataOrganize$set('public', 'makeComponents', function(spatial, intercepts,
             
           }
           
+          }
           speciesSpat <- c(unlist(speciesOne), unlist(speciesOther))
           
           
         }
-        
+      }
       }
       #else speciesSpat <- paste0(species,'_spatial(main = coordinates, model = speciesModel)', collapse = ' + ') #change this to speciesModel
       
@@ -614,7 +664,7 @@ dataOrganize$set('public', 'makeComponents', function(spatial, intercepts,
   
   if (!is.null(covariatenames)) {
     
-    if (!is.null(species)) {
+    if (!is.null(species) && speciesenvironment) {
       
       speciesCovs <- apply(expand.grid(paste0(species,'_'), covariatenames), MARGIN = 1, FUN = paste0,collapse = '')
       speciesCovClass <- rep(covariateclass, each = length(species))
@@ -650,7 +700,23 @@ dataOrganize$set('public', 'makeComponents', function(spatial, intercepts,
   
   if (intercepts) {
     
-    if (!is.null(species)) int <- paste0(species, '_intercept(1)')
+    if (!is.null(species)) {
+      
+      if (is.null(speciesintercept)) {
+        
+        int <- paste0(datanames, '_intercept(1)')
+        
+      } else if (speciesintercept) {
+          
+        int <- paste0(speciesname, '_intercepts(main = ', speciesname, ', model = "iid")')
+        
+      } else {
+        
+        int <- paste0(species, '_intercept(1)')
+        
+        }
+      
+    }
     else int <- paste0(datanames, '_intercept(1)')
     
     if (!is.null(marks)) intMarks <- paste0(marks, '_intercept(1)')
