@@ -53,7 +53,7 @@ pointCovs <- 'pointcov'
 speciesName <- 'species'
 markSpatial <- TRUE
 marksIntercept <- TRUE
-speciesSpatial <- TRUE
+speciesSpatial <- 'copy'
 temporalName <- 'temp'
 temporalModel <- deparse(list(model = 'ar1'))
 copyModel = deparse1(list(beta = list(fixed = FALSE)))
@@ -72,6 +72,7 @@ test_that('dataSDMs initialize works as expected.', {
                        Inlamesh = mesh, speciesspatial = speciesSpatial,
                        responsepa = responsePA,
                        trialspa = trialName,
+                       speciesindependent = FALSE,
                        responsecounts = responseCounts,
                        marksnames = markNames,
                        trialsmarks = markTrial,
@@ -80,6 +81,8 @@ test_that('dataSDMs initialize works as expected.', {
                        marksspatial = markSpatial,
                        marksintercept = marksIntercept,
                        spatialcovariates = cov,
+                       speciesintercept = FALSE,
+                       speciesenvironment = TRUE,
                        speciesname = speciesName,
                        ips = iPoints, copymodel = copyModel,
                        spatial = 'shared', temporal = temporalName, 
