@@ -2,40 +2,44 @@ test_that('intModel is able to initialize a dataSDM object as well as correctly
   skip_on_cran()
   
   ##Set up arbitrary data
-  projection <- CRS('+proj=tmerc')
+  projection <- '+proj=tmerc'
+  
+  #Make random shape to generate points on
   x <- c(16.48438,  17.49512,  24.74609, 22.59277, 16.48438)
   y <- c(59.736328125, 55.1220703125, 55.0341796875, 61.142578125, 59.736328125)
   xy <- cbind(x, y)
-  
-  Poly = Polygon(xy)
-  Poly = Polygons(list(Poly),1)
-  SpatialPoly = SpatialPolygons(list(Poly), proj4string = projection)
+  SpatialPoly <- st_sfc(st_polygon(list(xy)), crs = projection)
   
   ##Old coordinate names
   #Make random points
   #Random presence only dataset
-  PO <- spsample(SpatialPoly, n = 100, 'random', CRSobs = projection)
+  PO <- st_as_sf(st_sample(SpatialPoly, 100, crs = projection))
+  st_geometry(PO) <- 'geometry'
   ##Add random variable
-  PO$numvar <- runif(n = nrow(PO@coords))
-  PO$factvar <- sample(x = c('a','b'), size = nrow(PO@coords), replace = TRUE)
-  PO$species <- sample(x = c('fish'), size = nrow(PO@coords), replace = TRUE)
+  PO$numvar <- runif(n = nrow(PO))
+  PO$factvar <- sample(x = c('a','b'), size = nrow(PO), replace = TRUE)
+  PO$species <- sample(x = c('fish'), size = nrow(PO), replace = TRUE)
   #Random presence absence dataset
-  PA <- spsample(SpatialPoly, n = 100, 'random', CRSobs = projection)
-  PA$PAresp <- sample(x = c(0,1), size = nrow(PA@coords), replace = TRUE)
+  PA <- st_as_sf(st_sample(SpatialPoly, 100, crs = projection))
+  st_geometry(PA) <- 'geometry'
+  PA$PAresp <- sample(x = c(0,1), size = nrow(PA), replace = TRUE)
   #Add trial name
-  PA$trial <- sample(x = c(1,2,3), size = nrow(PA@coords), replace = TRUE)
-  PA$pointcov <- runif(n = nrow(PA@coords))
-  PA$binommark <- sample(x = 2:5, size = nrow(PA@data), replace = TRUE)
-  PA$marktrial <- sample(x = 0:1, size = nrow(PA@data), replace = TRUE)
-  PA$species <- sample(x = c('bird'), nrow(PA@data), replace = TRUE)
+  PA$trial <- sample(x = c(1,2,3), size = nrow(PA), replace = TRUE)
+  PA$pointcov <- runif(n = nrow(PA))
+  PA$binommark <- sample(x = 2:5, size = nrow(PA), replace = TRUE)
+  PA$marktrial <- sample(x = 0:1, size = nrow(PA), replace = TRUE)
+  PA$species <- sample(x = c('bird'), nrow(PA), replace = TRUE)
   mesh <- INLA::inla.mesh.2d(boundary = INLA::inla.sp2segment(SpatialPoly), 
-                             max.edge = 2)
+                             max.edge = 2, crs = inlabru::fm_crs(projection))
   #iPoints <- inlabru::ipoints(samplers = SpatialPoly, domain = mesh)
-  iPoints <- inlabru::ipoints(samplers = SpatialPoly)
+  iPoints <- inlabru::fm_int(samplers = SpatialPoly, domain = mesh)
   ##Make PA a data.frame object
+  PA$long <- st_coordinates(PA)[,1]
+  PA$lat <- st_coordinates(PA)[,2]
+  st_geometry(PA) <- NULL
   PA <- data.frame(PA)
   
-  coordnames <- colnames(PO@coords)
+  coordnames <- c('long', 'lat')
   responseCounts <- 'count'
   responsePA <- 'PAresp'
   trialName <- 'trial'
